﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameManager : MonoBehaviour
{
    PlayerController _player;
    SpawnManager _trashSpawner;
    CameraManager _mainCamera;

    public Animator GuiAnimator;
    public Text Score;

    public GameObject CarsShop;
    public GameObject SkinShop;

    // Start is called before the first frame update
    void Start()
    {
        try {
            _player = GameObject.FindGameObjectWithTag("Player").GetComponent<PlayerController>();
        }
        catch {
            Debug.LogError("Player not found! Please, add Player to scene for correct gameplay!");
        }

        try {
            _mainCamera = GameObject.FindGameObjectWithTag("MainCamera").GetComponent<CameraManager>();
        }
        catch {
            Debug.LogError("Camera Manager not found! Please, add Camera Manager to scene for correct gameplay!");
        }

        try {
            _trashSpawner = GameObject.FindWithTag("Respawn").GetComponent<SpawnManager>();
        }
        catch {
            Debug.LogError("EnemySpawner not found! Please, add Trash Spawner to scene for correct gameplay!");
        }

        StartCoroutine(SetScore());
    }

    private void FixedUpdate() {
        if (Input.GetKeyDown(KeyCode.Q))
            StopGame();

        if (Input.GetKeyDown(KeyCode.I))
            GoToShop();

        if (Input.GetKeyDown(KeyCode.S))
            StartGame();
        // -------------------------------------------
        if (Input.GetKeyDown(KeyCode.V))
            ChangePlayerSkin(1);

        if (Input.GetKeyDown(KeyCode.B))
            ChangePlayerSkin(2);

        if (Input.GetKeyDown(KeyCode.N))
            ChangePlayerSkin(3);

        if (Input.GetKeyDown(KeyCode.M))
            ChangePlayerSkin(4);
        
        if (Input.GetKeyDown(KeyCode.C))
            ChangePlayerSkin(5);

        // -------------------------------------------
        if (Input.GetKeyDown(KeyCode.F))
            ChangePlayerSkin(6);

        if (Input.GetKeyDown(KeyCode.G))
            ChangePlayerSkin(7);

        if (Input.GetKeyDown(KeyCode.H))
            ChangePlayerSkin(8);

        if (Input.GetKeyDown(KeyCode.J))
            ChangePlayerSkin(9);
        
        if (Input.GetKeyDown(KeyCode.K))
            ChangePlayerSkin(10);

        // -------------------------------------------
        if (Input.GetKeyDown(KeyCode.Escape)) 
            Application.Quit(); 
    }

    private IEnumerator SetScore() {
        while (true) {
            if (_player != null)
                Score.text = _player.TotalScore.ToString();
            yield return new WaitForEndOfFrame();
        }
    }

    public void ShowCars() {
        CarsShop.SetActive(true);
        SkinShop.SetActive(false);
    }

    public void ShowSkins() {
        CarsShop.SetActive(false);
        SkinShop.SetActive(true);
    }

    public void ChangePlayerSkin(int skinIndex) {
        Debug.Log("Changing player skin...");
        if (_player != null)
            _player.ChangePlayerSkin(skinIndex);
    }

    public void StartGameUI() {
        GuiAnimator.SetBool("shop_ON", false);
        GuiAnimator.SetBool("game_ON", true);
    }

    public void GoToShopUI() {
        GuiAnimator.SetBool("shop_ON", true);
        GuiAnimator.SetBool("game_ON", false);
    }

    public void StartGame() {
        Debug.Log("Moving camera to start point. Start point index: 0");
        if (_mainCamera != null)
            _mainCamera.MoveCameraToPoint(0);

        // do not change the order of next lines 
        // because player will be not able to move in game
        StartGameUI();
        MovePlayerPositionAndDisableMoving(_player.InGamePosition);
        ChangeScaleForPlayer(_player.InGameScale);
        LoadPlayerScoreFromSaved();
        AllowPlayerMoves();
        StartSpawningTrash();
    }

    public void StopGame() {
        Debug.Log("Moving camera to start point. Start point index: 0");
        if (_mainCamera != null)
            _mainCamera.MoveCameraToPoint(0);

        // do not change the order of next lines 
        // because player will be not able to move in game
        DisableSpawnAndClearAllTrash();
        MovePlayerPositionAndDisableMoving(_player.InGamePosition);
        ChangeScaleForPlayer(_player.InShopScale);
        SavePlayerScore();
    }
    
    public void ContinueGame()
    {
        StartGameUI();
        MovePlayerPositionAndDisableMoving(_player.InGamePosition);
        ChangeScaleForPlayer(_player.InGameScale);
        LoadPlayerScoreFromSaved();
        AllowPlayerMoves();

        Debug.Log("Moving camera to start point. Start point index: 0");
        if (_mainCamera != null)
            _mainCamera.MoveCameraToPoint(0);

        StartSpawningTrash();
    }

    public void GoToShop() {
        SavePlayerScore();
        GoToShopUI();
        MovePlayerPositionAndDisableMoving(_player.ShopPosition);
        ChangeScaleForPlayer(_player.InShopScale);

        Debug.Log("Moving camera to menu point. Menu point index: 1");
        if (_mainCamera != null)
            _mainCamera.MoveCameraToPoint(1);

        DisableSpawnAndClearAllTrash();
    }

    public void ChangeScaleForPlayer(Vector3 scale) {
        Debug.Log("Changing player scale to correct");
        if (_player != null) 
            _player.ChangePlayerScale(scale);
    }

    public void AllowPlayerMoves() {
        Debug.Log("Allowing player to move...");
            _player.AllowPlayerMove();
    }

    void MovePlayerPositionAndDisableMoving(Vector3 position){
        Debug.Log("Reseting player's position...");
        if (_player != null) {
            _player.DisablePlayerMove();
            _player.MovePlayerToPosition(position);
        }
    }

    public void StartSpawningTrash() {
        Debug.Log("Starting to spawn enemies!");
        _trashSpawner.StartSpawning();
    }

    public void DisableSpawnAndClearAllTrash() {
        Debug.Log("Stoping trash spawning...");
        if (_trashSpawner != null)
            _trashSpawner.StopSpawning();
        Debug.Log("Starting process of cleaning...");

        var allFreeTrahs = GameObject.FindGameObjectsWithTag("Trash");
        int trahsCounter = 0;
        foreach (var trash in allFreeTrahs) {
            Destroy(trash);
            trahsCounter++;
        }

        var allLandedTrash = GameObject.FindGameObjectsWithTag("TrashOnEarth");
        foreach (var trash in allLandedTrash) {
            Destroy(trash);
            trahsCounter++;
        }

        Debug.Log("All trash cleared! Total trash was: " + trahsCounter);
    }

    public void SavePlayerScore() {
        Debug.Log("Saving player score...");
        if (_player != null)
            PlayerPrefs.SetInt("PlayerScore", _player.TotalScore);
    }

    public void LoadPlayerScoreFromSaved() {
        Debug.Log("Loading player score...");
        if (_player != null && PlayerPrefs.HasKey("PlayerScore"))
            _player.TotalScore = PlayerPrefs.GetInt("PlayerScore");
    }

    public void BuySkin(int skinIndex) {
        var price = 0;
        Debug.Log("Buying a sking " + skinIndex + " with price " + price);
        if (_player != null) {
            _player.TotalScore -= price;
            ChangePlayerSkin(skinIndex);
            SavePlayerScore();
        }        
    }
}
