﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class SpawnManager : MonoBehaviour {
    public GameObject trashPrefab;
    private System.Random rand = new System.Random();

    private float _lvlRate = 3.0f;
    private float _canLvlUp = 0.0f;
    private float _generatingLvl = 1.0f;
    private float _speedLvl = 12.0f;

    private Coroutine _spawningProcess;
    private Coroutine _spawningCloudOfTrash;

    public Sprite[] sprites;

    public void StartSpawning() {
        Debug.Log("Setting default values for spawn parameters...");
        _generatingLvl = 1f;
        _speedLvl = 8f;
        _canLvlUp = 0.0f;
        Debug.Log("Starting spawn process in enemy manager...");
        _spawningProcess = StartCoroutine(GenarateRoutine());
        _spawningCloudOfTrash = StartCoroutine(GenarateCloud());
    }

    public void StopSpawning() {
        Debug.Log("Deactivating trash spawn process...");
        if (_spawningProcess != null) {
            StopCoroutine(_spawningProcess);
            StopCoroutine(_spawningCloudOfTrash);
        }
        _spawningProcess = null;
        _spawningCloudOfTrash = null;
        Debug.Log("Trahs spawn stoped!");
    }

    private IEnumerator GenarateRoutine() {
        int orderInLayer = 0;
        while (true) {
            Vector3 startCoord = new Vector3((float)rand.NextDouble(), 1.3f, 0);
            Vector3 aimCoord = new Vector3((float)rand.NextDouble(), -0.2f, 0);

            var trash = Instantiate(trashPrefab, Camera.main.ViewportToWorldPoint(startCoord), Quaternion.identity);
            var trashEntity = trash.GetComponent<TrashAction>();
            var trashSpRen = trash.GetComponent<SpriteRenderer>();
            trashEntity.aimPoint = Camera.main.ViewportToWorldPoint(aimCoord);
            trashSpRen.sortingOrder = orderInLayer;
            orderInLayer += 1;
            trashEntity.rotation = rand.Next(-20, 20);
            var lvl = rand.Next(0, sprites.Length);
            trashSpRen.sprite = sprites[lvl];
            trashEntity.score = (int)Math.Ceiling(lvl/4.0f);
            if (trashEntity.score == 0) trashEntity.score = 1;

            Debug.Log("Score of trash: " + trashEntity.score);

            if (rand.Next(0, 2) == 0) {
                trash.layer = 8;
            } else {
                trash.layer = 9;
            }

            if (Time.time > _canLvlUp) {
                _canLvlUp = Time.time + _lvlRate;
                _generatingLvl += 1.5f;
                _speedLvl += 1.0f;
            }
            trashEntity.speed = _speedLvl / 8.0f + rand.Next(1, 5);
            yield return new WaitForSeconds(5.0f/_generatingLvl);
        }
    }

    private IEnumerator GenarateCloud() {
        while (true) {
            Vector3 startCoordForCloud = new Vector3(1f, rand.Next(93, 100)/100f, 0f);
            Vector3 aimCoordForCloud = new Vector3(0f, rand.Next(93, 100)/100f, 0f);

            var cloudTrash = Instantiate(trashPrefab, Camera.main.ViewportToWorldPoint(startCoordForCloud), Quaternion.identity);
            var cloudTrashEntity = cloudTrash.GetComponent<TrashAction>();
            var trashSpRen = cloudTrash.GetComponent<SpriteRenderer>();
            cloudTrashEntity.aimPoint = Camera.main.ViewportToWorldPoint(aimCoordForCloud);
            
            trashSpRen.sortingLayerName = "Background";
            cloudTrash.gameObject.GetComponent<Collider2D>().isTrigger = true;

            cloudTrashEntity.rotation = rand.Next(-80, 80);

            trashSpRen.sprite = sprites[rand.Next(0, sprites.Length)];

            var color = trashSpRen.material.color;
            trashSpRen.color = new Color(color.r * 0.6f, color.g * 0.6f, color.b * 0.6f, color.a);

            cloudTrashEntity.speed = 20.0f / _speedLvl + rand.Next(1, 3);
            yield return new WaitForSeconds(_generatingLvl/40.0f);
        }
    }
}
