﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;


public class TrashAction : MonoBehaviour {
    private Rigidbody2D _rb;
    private SpriteRenderer _spRend;
    private float destroyDistace;
    private Coroutine fallRoutine;
    private Coroutine scaleToSmall;
    private System.Random rand = new System.Random();
    private bool touchToEarth = false;

    public float speed = 1.0f;
    public Vector3 aimPoint;
    public float rotation = 0;
    public int score;
    
    void Start() {
        transform.position = new Vector3(transform.position.x, transform.position.y, 0);
        aimPoint = new Vector3(aimPoint.x, aimPoint.y, 0);
        destroyDistace = Vector3.Distance(
            new Vector3(),
            Camera.main.ViewportToWorldPoint(new Vector3(0, 1.5f)));

        try {
            _rb = GetComponent<Rigidbody2D>();
            _spRend = GetComponent<SpriteRenderer>();
        } catch {
            Debug.Log("Error with rb or spRend!");
        }

        fallRoutine = StartCoroutine(FallRoutine());
        StartCoroutine(ChackDestroyRoutine());
    }

    private IEnumerator ChackDestroyRoutine() {
        while (true) {
            if (Vector3.Distance(transform.position, new Vector3()) > destroyDistace) {
                Destroy(this.gameObject);
                break;
            } else if (Vector3.Distance(transform.position, aimPoint) < 0.001f) {
                Destroy(this.gameObject);
            } else if (Camera.main.WorldToViewportPoint(transform.position).x > 1.1 ||
                Camera.main.WorldToViewportPoint(transform.position).x < -0.1) {
                Destroy(this.gameObject);
            }
            yield return new WaitForSeconds(Time.deltaTime);
        }
    }

    private IEnumerator FallRoutine() {
        _rb.AddTorque(rotation);
        while (true) {
            transform.position = Vector3.MoveTowards(transform.position, aimPoint, speed * Time.deltaTime);
            yield return new WaitForSeconds(Time.deltaTime);
        }
    }

    private IEnumerator ScaleToSmall(GameObject collision) {
        var endTime = Time.time + rand.Next(1, 22)/100.0f;
        while (true) {
            transform.position = Vector3.MoveTowards(transform.position, collision.gameObject.transform.position, speed * Time.deltaTime);
            transform.localScale -= transform.localScale * 0.2f;
            if (Time.time > endTime) {
                StickToEarth(collision);
                break;
            }
            yield return new WaitForSeconds(Time.deltaTime * speed);
        }
    }

    private void OnTriggerEnter2D(Collider2D collision) {
        if (collision.gameObject.tag == "Earth" && collision.gameObject != null && touchToEarth == false) {
            _rb.gravityScale = 0;
            touchToEarth = true;
            scaleToSmall = StartCoroutine(ScaleToSmall(collision.gameObject));
            //_rb.AddForce(collision.transform.position, ForceMode2D.Impulse);
        }
    }

    private void OnCollisionEnter2D(Collision2D collision) {
        float force = 1.5f;
        if (collision.gameObject.tag == "Trash" && collision.gameObject != null &&
            Camera.main.WorldToViewportPoint(transform.position).y < 1.0 && touchToEarth == false) {
            Vector3 direction = new Vector3(collision.contacts[0].point.x, collision.contacts[0].point.y) - collision.gameObject.transform.position;
            direction.Normalize();
            StopCoroutine(fallRoutine);
            _rb.AddForce(direction * force, ForceMode2D.Impulse);
            _rb.gravityScale = 1;
        } else if (collision.gameObject.tag == "Earth" && collision.gameObject != null && touchToEarth == true) {
            StickToEarth(collision.gameObject);
        }
    }

    private void StickToEarth(GameObject collision) { //Nani???
        StopAllCoroutines();
        _rb.isKinematic = true;
        _rb.freezeRotation = true;
        _rb.velocity = new Vector2();
        //Destroy(_rb);
        GetComponent<Collider2D>().isTrigger = true;
        transform.SetParent(collision.gameObject.transform);
        gameObject.tag = "TrashOnEarth";
    }
}
